#include "bootpack.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef enum {
	Letter, Other, TEOF, TNull
} Type;

typedef struct {
	char text[32];
	Type type;
} Token;

static Token token;
static struct FILEINFO *finfo;
static int *ctype;
static unsigned char *filebuf;
static unsigned char *fp;
static struct MEMMAN *memman = (struct MEMMAN *) MEMMAN_ADDR;

int isspace(int c)
{
	return (c == ' ' || c == '\t' || c == '\n' || c == '\r');
}

static int nextChar(void)
{
	static int c;
	c = *fp; fp++;
	if (c == 0)
		return -1;
	return c;
}

static int nextChar_dch(void)
{
	int c = ' ';
	while (isspace(c))
		c = nextChar();
	return c;
}

static void init_token()
{
	extern int *tmp_fat;
	int i, fsize;
	ctype = (unsigned int *) memman_alloc_4k(memman, 0xff * sizeof(int));
	fsize = finfo->size;
	filebuf = (unsigned char *) memman_alloc_4k(memman, fsize);
	filebuf = file_loadfile2(finfo->clustno, &fsize, tmp_fat);
	filebuf[fsize] = 0;
	fp = filebuf;
	for (i = 0; i < 0xff; i++)	ctype[i] = Other;
	for (i='0'; i <= '9'; i++)	ctype[i] = Letter;
	for (i='A'; i <= 'Z'; i++)	ctype[i] = Letter;
	for (i='a'; i <= 'z'; i++)	ctype[i] = Letter;
    ctype['_'] = Letter;
    ctype['<'] = Letter;
    ctype['>'] = Letter;
	return;
}

static void GetToken()
{
	int i, ch;
	char text[32];

	if ((ch = nextChar_dch()) == -1) {
		token.type = TEOF;
		return;
	}

	switch (ctype[ch]) {
		case Letter:
			for (i = 0; ctype[ch] == Letter; i++) {
				if (i >= 32 || ch == -1)
					break;
				text[i] = ch;
				ch = nextChar();
			}
			text[i] = 0;
			strcpy(token.text, text);
			break;
		default:
			token.type = TNull;
			break;
	}

	return;
}

int Init_iniFile(char *filename)
{
	finfo = file_search(filename, (struct FILEINFO *) (ADR_DISKIMG + 0x002600), 224);
	if (finfo == 0) {
		return 0;
	}
	init_token();
	return 1;
}

InitData Read_iniFile(void)
{
	static InitData i;

	GetToken();
	strcpy(i.text, token.text);
	i.ret = (token.type == TNull || token.type == TEOF);

	return i;
}

int End_iniFile(void)
{
	memman_free_4k(memman, (int) filebuf, finfo->size);
	return 1;
}

